﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using GeoAPI.Geometries;
using HeuristicLab.Persistence.Core;
using HeuristicLab.Persistence.Interfaces;
using NetTopologySuite.IO;
using System;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.BioBoost.Persistence {
  
  public class GeometrySerializer : ICompositeSerializer {

    #region ICompositeSerializer Members

    public int Priority {
      get { return 300; }
    }

    public bool CanSerialize(Type type) {
      return type.GetInterfaces().Contains(typeof (IGeometry));
    }

    public string JustifyRejection(Type type) {
      return "type does not implement interface IGeometry";
    }

    public object CreateInstance(Type type, IEnumerable<Tag> metaInfo) {
      return new WKBReader().Read((byte[]) metaInfo.First().Value);
    }

    public IEnumerable<Tag> CreateMetaInfo(object obj) {
      var g = obj as IGeometry;
      return new[] {new Tag("WKB", new WKBWriter().Write(g))};
    }

    public IEnumerable<Tag> Decompose(object obj) {
      // object was fully decomposed in CreateMetaInfo
      return new Tag[0];
    }


    public void Populate(object instance, IEnumerable<Tag> tags, Type type) {
      // object was fully populated in CreateInstance
    }


    #endregion
  }
}
